@@ -1,8 +1,9 @@
 # SPL_CGT_INCLUDE: ../pyspltuple2tuple.cgt
-# SPL_CGT_INCLUDE: ../../opt/python/codegen/py_splTupleCheckForBlobs.cgt
-# SPL_CGT_INCLUDE: ../pyspltuple.cgt
 # SPL_CGT_INCLUDE: ../pyspltuple2value.cgt
+# SPL_CGT_INCLUDE: ../../opt/python/codegen/py_pyTupleTosplTuple.cgt
 # SPL_CGT_INCLUDE: ../pyspltuple_constructor.cgt
+# SPL_CGT_INCLUDE: ../pyspltuple.cgt
+# SPL_CGT_INCLUDE: ../../opt/python/codegen/py_splTupleCheckForBlobs.cgt
 # SPL_CGT_INCLUDE: ../pyspltuple2dict.cgt
 
 package Aggregate_cpp;
@@ -81,6 +82,11 @@ sub main::generate($$) {
    print 'MY_OPERATOR_SCOPE::MY_OPERATOR::MY_OPERATOR() :', "\n";
    print '   funcop_(NULL),', "\n";
    print '   pyInStyleObj_(NULL),', "\n";
+   if ($pyoutstyle eq 'dict') {
+   print '    ', "\n";
+   print '   pyOutNames_0(NULL),', "\n";
+   }
+   print '    ', "\n";
    print '   loads(NULL),', "\n";
    print '   occ_(-1),', "\n";
    print '   window_(';
@@ -180,6 +186,13 @@ sub main::generate($$) {
    print '    {', "\n";
    print '    SplpyGIL lock;', "\n";
    print '    ';
+   if ($pyoutstyle eq 'dict') {
+   print "\n";
+   print '    pyOutNames_0 = Splpy::pyAttributeNames(getOutputPortAt(0));', "\n";
+   print '    ';
+   }
+   print '    ', "\n";
+   print '    ';
    if ($pystyle eq 'pickle'){
    print "\n";
    print '    loads = SplpyGeneral::loadFunction("pickle", "loads");', "\n";
@@ -234,6 +247,16 @@ sub main::generate($$) {
    print "\n";
    print 'MY_OPERATOR_SCOPE::MY_OPERATOR::~MY_OPERATOR() ', "\n";
    print '{', "\n";
+   print '  ';
+   if ($pyoutstyle eq 'dict') {
+   print ' ', "\n";
+   print '  {', "\n";
+   print '    SplpyGIL lock;', "\n";
+   print '    Py_CLEAR(pyOutNames_0);', "\n";
+   print '  }', "\n";
+   print '  ';
+   }
+   print '  	', "\n";
    print '  delete funcop_;', "\n";
    print "\n";
    print '#if SPLPY_PARTITION_IS_STATEFUL == 1', "\n";
@@ -254,6 +277,7 @@ sub main::generate($$) {
    print '  ';
     } 
    print "\n";
+   print '  ', "\n";
    print '}', "\n";
    print "\n";
    print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::prepareToShutdown() ', "\n";
@@ -555,16 +579,40 @@ sub main::generate($$) {
    print '    try {', "\n";
    print '      SplpyGIL lock;', "\n";
    print '      SPLAPPTRC(L_DEBUG, "aggregation: call", SPLPY_SH_ASPECT);', "\n";
+   print '      ';
+   if ($pyoutstyle eq 'dict') {
+   print "\n";
+   print '	    PyObject * ret = streamsx::topology::Splpy::pyTupleMap(funcop_->callable(), items);', "\n";
+   print '	    if (ret == NULL)', "\n";
+   print '		  return;', "\n";
+   print '	    if (PyTuple_Check(ret)) {', "\n";
+   print '		  fromPyTupleToSPLTuple(ret, otuple);', "\n";
+   print '		  Py_DECREF(ret);', "\n";
+   print '	    } else if (PyDict_Check(ret)) {', "\n";
+   print '		  fromPyDictToSPLTuple(ret, otuple);', "\n";
+   print '		  Py_DECREF(ret);', "\n";
+   print '	    } else {', "\n";
+   print '		  throw SplpyGeneral::generalException("submit",', "\n";
+   print '		   "Fatal error: Value submitted must be a Python tuple or dict.");', "\n";
+   print '	    }', "\n";
+   print '	  SPLAPPTRC(L_DEBUG, "aggregation: submit", SPLPY_SH_ASPECT);', "\n";
+   print '      submit(otuple, 0);', "\n";
+   print '      ';
+    } else { 
+   print "\n";
    print '      if (SPLPY_AGGREGATE(funcop_->callable(), items,', "\n";
    print '        otuple.get_';
    print $model->getOutputPortAt(0)->getAttributeAt(0)->getName();
-   print '(), occ_)){  ', "\n";
+   print '(), occ_)){', "\n";
    print '        SPLAPPTRC(L_DEBUG, "aggregation: submit", SPLPY_SH_ASPECT);', "\n";
    print '        submit(otuple, 0);', "\n";
    print '      } ', "\n";
    print '      else {', "\n";
    print '        SPLAPPTRC(L_DEBUG, "aggregation: fail", SPLPY_SH_ASPECT);', "\n";
-   print '      }	    ', "\n";
+   print '      }	   ', "\n";
+   print '      ';
+   }
+   print "\n";
    print '    } catch (const streamsx::topology::SplpyExceptionInfo& excInfo) {', "\n";
    print '      SPLAPPTRC(L_DEBUG, "aggregation: caught", SPLPY_SH_ASPECT);', "\n";
    print '      SPLPY_OP_HANDLE_EXCEPTION_INFO_GIL(excInfo);', "\n";
@@ -675,6 +723,246 @@ sub main::generate($$) {
    print '  }', "\n";
    print "\n";
    print '#endif', "\n";
+   print "\n";
+     if ($pyoutstyle eq 'dict') {
+   	my %cpp_tuple_types;  
+       # In this case we don't want the function that
+       # converts the Python tuple to an SPL tuple to
+       # copy attributes from the input port
+       my $iport;
+   
+       my $oport = $model->getOutputPortAt(0);
+       my $oport_submission = 0;
+       my $otupleType = $oport->getSPLTupleType();
+       my @onames = SPL::CodeGen::Type::getAttributeNames($otupleType);
+       my @otypes = SPL::CodeGen::Type::getAttributeTypes($otupleType);
+   
+   print "\n";
+   print '// Create member function that converts Python tuple to SPL tuple', "\n";
+   # Generates functions in an operator that converts a Python
+   # tuple to an SPL tuple for a given port and optional to
+   # submit the tuple.
+   #
+   # $oport must be set on entry to required output port
+   # $oport_submission must be set on entry to generate submission methods.
+   # $iport can be set to automatically copy input attributes to
+   # output attributes when the Python tuple does not supply a value.
+   
+     my $itypeparam = "";
+     my $itypearg = "";
+     if (defined $iport) {
+        $itypeparam = ", " . $iport->getCppTupleType() . " const & ituple";
+        $itypearg = ", ituple";
+     }
+   print "\n";
+   print "\n";
+    if ($oport_submission) { 
+   print "\n";
+   print ' ', "\n";
+   print '// Python tuple to SPL tuple with submission to a port', "\n";
+   print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::fromPythonToPort';
+   print $oport->getIndex();
+   print '(PyObject *pyTuple, ';
+   print $oport->getCppTupleType();
+   print ' & otuple ';
+   print $itypeparam;
+   print ') {', "\n";
+   print "\n";
+   print '  try {', "\n";
+   print '    MY_OPERATOR_SCOPE::MY_OPERATOR::fromPyTupleToSPLTuple(pyTuple, otuple ';
+   print $itypearg;
+   print ');', "\n";
+   print '  } catch (const streamsx::topology::SplpyExceptionInfo& excInfo) {', "\n";
+   print '    SPLPY_OP_HANDLE_EXCEPTION_INFO(excInfo);', "\n";
+   print '    return;', "\n";
+   print '  }', "\n";
+   print "\n";
+   print '  STREAMSX_TUPLE_SUBMIT_ALLOW_THREADS(otuple, ';
+   print $oport->getIndex();
+   print ');', "\n";
+   print '}', "\n";
+   print "\n";
+   print '// Python dict to SPL tuple with submission to a port.', "\n";
+   print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::fromPythonDictToPort';
+   print $oport->getIndex();
+   print '(PyObject *pyDict, ';
+   print $oport->getCppTupleType();
+   print ' & otuple ';
+   print $itypeparam;
+   print ') {', "\n";
+   print "\n";
+   print '  try {', "\n";
+   print '    MY_OPERATOR_SCOPE::MY_OPERATOR::fromPyDictToSPLTuple(pyDict, otuple ';
+   print $itypearg;
+   print ');', "\n";
+   print '  } catch (const streamsx::topology::SplpyExceptionInfo& excInfo) {', "\n";
+   print '    SPLPY_OP_HANDLE_EXCEPTION_INFO(excInfo);', "\n";
+   print '    return;', "\n";
+   print '  }', "\n";
+   print "\n";
+   print '  STREAMSX_TUPLE_SUBMIT_ALLOW_THREADS(otuple, ';
+   print $oport->getIndex();
+   print ');', "\n";
+   print '}', "\n";
+   print "\n";
+   }
+   print "\n";
+   print "\n";
+   # Ensure we generate function only once for each tuple type
+   my $otype = $oport->getCppTupleType();
+   if (! exists $cpp_tuple_types{$otype}) {
+       $cpp_tuple_types{$otype} = 1;
+   print "\n";
+   print "\n";
+   print '// Python tuple to SPL tuple , conversion only', "\n";
+   print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::fromPyTupleToSPLTuple(PyObject *pyTuple, ';
+   print $oport->getCppTupleType();
+   print ' & otuple  ';
+   print $itypeparam;
+   print ') {', "\n";
+   print "\n";
+   print '  Py_ssize_t frs = PyTuple_GET_SIZE(pyTuple); ', "\n";
+   print '    ', "\n";
+     if (defined $iport) {
+       print 'bool setAttr = false;';
+     }
+   
+     for (my $ai = 0; $ai < $oport->getNumberOfAttributes(); ++$ai) {
+       
+       my $attribute = $oport->getAttributeAt($ai);
+       my $name = $attribute->getName();
+       my $atype = $attribute->getSPLType();
+       splToPythonConversionCheck($atype);
+       
+       if (defined $iport) {
+                print 'setAttr = false;';
+       }
+   print "\n";
+   print '    if (';
+   print $ai;
+   print ' < frs) {', "\n";
+   print '         // Value from the Python function', "\n";
+   print '         PyObject *pyAttrValue = PyTuple_GET_ITEM(pyTuple, ';
+   print $ai;
+   print ');', "\n";
+   print '         if (!SplpyGeneral::isNone(pyAttrValue)) {', "\n";
+   print '                  streamsx::topology::pySplValueFromPyObject(', "\n";
+   print '                               otuple.get_';
+   print $name;
+   print '(), pyAttrValue);', "\n";
+       if (defined $iport) {
+                print 'setAttr = true;';
+       }
+   print "\n";
+   print '      }', "\n";
+   print '   }', "\n";
+       if (defined $iport) {
+       
+       # Only copy attributes across if they match on name and type,
+       # or on name and input type T and output type optional<T>
+       my $matchInputAttr = $iport->getAttributeByName($name);
+       if (defined $matchInputAttr) {
+           my $inputType = $matchInputAttr->getSPLType();
+           if (($inputType eq $atype) ||
+               (hasOptionalTypesSupport() &&
+                SPL::CodeGen::Type::isOptional($atype) &&
+                ($inputType eq
+                 SPL::CodeGen::Type::getUnderlyingType($atype)))) {
+   print "\n";
+   print '    if (!setAttr) {', "\n";
+   print '      // value from the input attribute', "\n";
+   print '      otuple.set_';
+   print $name;
+   print '(ituple.get_';
+   print $name;
+   print '());', "\n";
+   print '    }', "\n";
+         }
+       }
+      }
+   print "\n";
+   print '         ', "\n";
+   }
+    
+   print "\n";
+   print "\n";
+   print '}', "\n";
+   print "\n";
+   print "\n";
+   print '// Python dict to SPL tuple , conversion only', "\n";
+   print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::fromPyDictToSPLTuple(PyObject *pyDict, ';
+   print $oport->getCppTupleType();
+   print ' & otuple  ';
+   print $itypeparam;
+   print ') {', "\n";
+   print "\n";
+   print '  Py_ssize_t available = PyDict_Size(pyDict); ', "\n";
+   print '    ', "\n";
+     if (defined $iport) {
+       print 'bool setAttr = false;';
+     }
+   
+     for (my $ai = $oport->getNumberOfAttributes() - 1; $ai >= 0; --$ai) {
+       
+       my $attribute = $oport->getAttributeAt($ai);
+       my $name = $attribute->getName();
+       my $atype = $attribute->getSPLType();
+       splToPythonConversionCheck($atype);
+       
+       if (defined $iport) {
+                print 'setAttr = false;';
+       }
+   print "\n";
+   print '    if (available > 0) {', "\n";
+   print '         // Value from the Python function', "\n";
+   print '         PyObject *pyAttrValue = PyDict_GetItem(pyDict, PyTuple_GET_ITEM(pyOutNames_';
+   print $oport->getIndex();
+   print ', ';
+   print $ai;
+   print '));', "\n";
+   print '         if (pyAttrValue != NULL) {', "\n";
+   print '             --available;', "\n";
+   print '             if (!SplpyGeneral::isNone(pyAttrValue)) {', "\n";
+   print '                  streamsx::topology::pySplValueFromPyObject(', "\n";
+   print '                               otuple.get_';
+   print $name;
+   print '(), pyAttrValue);', "\n";
+       if (defined $iport) {
+                print 'setAttr = true;';
+       }
+   print "\n";
+   print '           }', "\n";
+   print '        }', "\n";
+   print '   }', "\n";
+       if (defined $iport) {
+       
+       # Only copy attributes across if they match on name and type
+       my $matchInputAttr = $iport->getAttributeByName($name);
+       if (defined $matchInputAttr) {
+          if ($matchInputAttr->getSPLType() eq $attribute->getSPLType()) {
+   print "\n";
+   print '    if (!setAttr) {', "\n";
+   print '      // value from the input attribute', "\n";
+   print '      otuple.set_';
+   print $name;
+   print '(ituple.get_';
+   print $name;
+   print '());', "\n";
+   print '    }', "\n";
+         }
+       }
+      }
+   print "\n";
+   print '         ', "\n";
+   }
+    
+   print "\n";
+   print '}', "\n";
+    } 
+   print "\n";
+   }
+   print "\n";
+   print "\n";
    print "\n";
    SPL::CodeGen::implementationEpilogue($model);
    print "\n";
